#!/bin/bash

echo "=== YACBA Docker Builder ==="

if ! command -v docker &> /dev/null; then
    echo "[-] Docker не установлен"
    echo "[*] Установите Docker: https://docs.docker.com/get-docker/"
    exit 1
fi

echo "[*] Сборка через Docker контейнер..."

docker run --rm -v "$(pwd)":/work -w /work ubuntu:20.04 bash -c '
apt-get update -qq
apt-get install -y nasm gcc binutils make

echo "[*] Сборка загрузчика..."
nasm -f bin bootloader.asm -o bootloader.bin

echo "[*] Сборка ядра..."
gcc -m32 -ffreestanding -nostdlib -nodefaultlibs -fno-builtin -fno-stack-protector -c kernel.c -o kernel.o

echo "[*] Линковка ядра..."
ld -m elf_i386 -T linker.ld kernel.o -o kernel.bin

echo "[*] Создание образа ОС..."
cat bootloader.bin kernel.bin > os.bin
truncate -s 1440K os.bin

echo "[+] Сборка завершена!"
ls -lh os.bin
'

if [ -f "os.bin" ]; then
    echo "[+] Образ os.bin готов!"
    echo "[*] Размер: $(ls -lh os.bin | awk '{print $5}')"
    echo "[*] Для создания флешки: sudo ./create_boot_usb.sh /dev/sdX"
else
    echo "[-] Ошибка сборки"
    exit 1
fi 